<?php
/*
Plugin Name: CryoKey
Description: Basic CryoKey Integration
Version: 2.1
Author: Authenticade LLC
License: MIT
*/

/**
 * Check for CryoKey credentials. If available, automatically
 * log in.
 */
function ck_init()
{
  $tid = @$_REQUEST['ckid'];
  if (isset($tid))
  {
    // Try CryoKey login first. Keep priority low (10) to ensure it
    // runs first. 3 arguments.
    add_filter('authenticate', 'ck_login', 10, 3);
    $user = wp_signon();
    if (!is_wp_error($user))
    {
      // Due to a bug in is_user_logged_in(), we have to explicitly
      // assign the current user after wp_signon() or else WordPress
      // doesn't think we're logged in (until the next page load).
      wp_set_current_user($user->ID);
      remove_filter('loginout', 'ck_link');
    }
    else
    {
      // Login failed, and we're not creating users.
      wp_safe_redirect(wp_login_url());
      exit;
    }
  }
  else
  {
    // We're not in the login module.
    if (is_user_logged_in())
    {
      remove_filter('loginout', 'ck_link');
    }
  }
}

/**
 * Show a button for CryoKey logins (usually in login form).
 */
function ck_button()
{
  $js = plugins_url('cryokey.js', __FILE__);
  $icon = plugins_url('ckicon.png', __FILE__);
  $service = wp_login_url();
  $ip = $_SERVER['REMOTE_ADDR'];
  $pass = site_url();
  echo "<p><script type='text/javascript' src='{$js}'></script><a href='javascript:ck_ticket(\"{$service}\", \"{$ip}\", \"{$pass}\");'><img src='{$icon}' width='16' height='16' alt='Log In Using CryoKey'/> Log In Using CryoKey</a></p>";
}

/**
 * Try to auto-fill the E-Mail address field (document ID
 * 'user_email').
 */
function ck_fill()
{
  // Note that this will override the POST value during registration.
  if (isset($_REQUEST['ckemail']))
    echo "<script type='text/javascript'>document.getElementById('user_email').value = '{$_REQUEST['ckemail']}';</script>";
}

/**
 * Attach the CryoKey Lite JavaScript file.
 */
function ck_scripts()
{
  wp_enqueue_script('cryokey', plugins_url('cryokey.js', __FILE__));
}

/**
 * Determine the current user based on certificate authentication. If
 * using CryoKey credentials, login/logout is meaningless. Therefore,
 * we change the login link to show the current user (or a
 * registration link for unregistered users).
 */
function ck_login($_user, $_username, $_password)
{
  $auto = ("1" == get_option('ck_auto_register')) ? true : false;
  if (isset($_GET['ckid']))
    $email = ck_redeem($_GET['ckid'], wp_login_url());
  if (empty($email))
    return new WP_Error('authentication failed', __("Couldn't verify credentials."));

  // If the user requested registration, add the user.
  if (false == get_user_by('email', $email))
  {
    if ($auto)
    {
      $name = ck_scramble(8);
      $registration = array(
        'user_login' => "CK{$email}",
        'user_pass' => wp_generate_password(),
        'user_email' => $email,
        'user_nicename' => $name,
        'display_name' => "CK{$name}",
        'nickname' => $name,
        'description' => "CryoKey Generated Registration"
      );

      //wp_create_user("CK{$email}", wp_generate_password(), $email);
      wp_insert_user($registration);
    }
  }

  // Check for existing user.
  $_user = get_user_by('email', $email);
  if ($_user)
  {
    // Change the current user.
    return $_user;
  }

  // We have valid CryoKey credentials for a user that doesn't exist.
  if (get_option('users_can_register'))
  {
    // If anyone can register, send the user to the registration
    // page instead.
    $arguments = array('action' => 'register', 'ckemail' => urlencode($email));
    wp_safe_redirect(add_query_arg($arguments, wp_login_url()));
    exit;
  }

  // Unknown user, and didn't create.
  return new WP_Error('authentication failed', __("No matching user."));
}

/**
 * This function will replace the login/logout link to display
 * a log in link. We log in using the CryoKey Lite mechanism.
 */
function ck_link($_input)
{
  $service = wp_login_url();
  $ip = $_SERVER['REMOTE_ADDR'];
  $pass = site_url();
  //$pass = get_permalink();
  return "<a href='javascript:ck_ticket(\"{$service}\", \"{$ip}\", \"{$pass}\");'>Log In</a>";
}

/**
 * Generate a random string of a given length.
 */
function ck_scramble($_length)
{
  $symbols = "abcdefghijklmnopqrstuvwxyz1234567890";
  $result = "";
  $available = strlen($symbols) - 1;
  for ($index = 0; $index < $_length; $index++)
  {
    $code = rand(0, $available);
    $result .= $symbols[$code];
  }
  return $result;
}
/**
 * Try to redeem a ticket. We return an E-Mail address on success,
 * or NULL on failure.
 */
function ck_redeem($_tid, $_service)
{
  return file_get_contents("https://www.cryokey.com/public/redeem.php?tid=" . urlencode($_tid) . "&from=" . urlencode($_SERVER['REMOTE_ADDR']) . "&to=" . urlencode($_service));
}

// Always check for CryoKey on init.
add_action('init', 'ck_init');

// Include CryoKey JavaScript.
add_action('wp_enqueue_scripts', 'ck_scripts');

// Remove these if the user logged in.
add_action('login_form', 'ck_button');
add_action('register_form', 'ck_fill');
add_filter('loginout', 'ck_link');

if (is_admin())
{
  // Add CryoKey options panel.
  add_action('admin_init', 'ck_admin');
  add_action('admin_menu', 'ck_menu');
}

////////////////////////////////
// CRYOKEY OPTIONS MENU SETUP //
////////////////////////////////
function ck_menu()
{
  add_options_page('CryoKey Options', 'CryoKey', 'manage_options', __FILE__, 'ck_options');
}
function ck_admin()
{
  register_setting('ck_options_group', 'ck_auto_register');
  add_settings_section('ck_main', 'CryoKey Main Options', 'ck_show_main', __FILE__);
  add_settings_field('ck_main_ar', 'Auto Register New Users', 'ck_show_main_ar', __FILE__, 'ck_main');
}
function ck_show_main()
{
  echo "Tune your CryoKey integration.";
}
function ck_show_main_ar($_arguments)
{
  if ("1" == get_option('ck_auto_register'))
    echo "<input type='checkbox' name='ck_auto_register' id='ck_main_ar' value='1' checked='checked'>Enabled</input>";
  else
    echo "<input type='checkbox' name='ck_auto_register' id='ck_main_ar' value='1'>Enabled</input>";
}
function ck_options()
{
  if (!current_user_can('manage_options'))
    wp_die(__('You do not have sufficient permissions to access this page.'));

?>
  <div class="wrap">
  <?php screen_icon(); ?>
  <h2>CryoKey Options</h2>
  <form method="post" action="options.php">
  <?php settings_fields('ck_options_group'); ?>
  <?php do_settings_sections(__FILE__); ?>
  <?php submit_button(); ?>
  </form>
  </div>
<?php
}

?>
